@@ -1,4 +1,3 @@
-
 # Copyright (C) 2019-2020 Intel Corporation
 #
 # SPDX-License-Identifier: MIT
@@ -7,6 +6,5 @@ import sys
 
 from datumaro.cli.__main__ import main
 
-
-if __name__ == '__main__':
+if __name__ == "__main__":
     sys.exit(main())
