@@ -2,24 +2,26 @@
 #
 # SPDX-License-Identifier: MIT
 
-from collections import Counter
-from enum import Enum, auto
-from itertools import zip_longest
-from typing import Union
 import logging as log
 import os
 import os.path as osp
 import warnings
+from collections import Counter
+from enum import Enum, auto
+from itertools import zip_longest
+from typing import Union
 
 import cv2
 import numpy as np
 
+from datumaro.components.media import Image
+
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")
     import tensorboardX as tb
 
+from datumaro.components.annotation import AnnotationType, LabelCategories
 from datumaro.components.dataset import IDataset
-from datumaro.components.extractor import AnnotationType, LabelCategories
 from datumaro.util import parse_str_enum_value
 from datumaro.util.image import save_image
 
@@ -32,12 +34,14 @@ class DiffVisualizer:
     DEFAULT_FORMAT = OutputFormat.simple
     _UNMATCHED_LABEL = -1
 
-    def __init__(self, comparator, save_dir: str,
-            output_format: Union[None, str, OutputFormat] = None):
+    def __init__(
+        self, comparator, save_dir: str, output_format: Union[None, str, OutputFormat] = None
+    ):
         self._cmp = comparator
 
-        self._output_format = parse_str_enum_value(output_format,
-            self.OutputFormat, default=self.DEFAULT_FORMAT)
+        self._output_format = parse_str_enum_value(
+            output_format, self.OutputFormat, default=self.DEFAULT_FORMAT
+        )
 
         self._save_dir = save_dir
 
@@ -45,7 +49,7 @@ class DiffVisualizer:
         os.makedirs(self._save_dir, exist_ok=True)
 
         if self._output_format is self.OutputFormat.tensorboard:
-            logdir = osp.join(self._save_dir, 'logs', 'diff')
+            logdir = osp.join(self._save_dir, "logs", "diff")
             self._file_writer = tb.SummaryWriter(logdir)
         elif self._output_format is self.OutputFormat.simple:
             self._label_diff_writer = None
@@ -71,15 +75,14 @@ class DiffVisualizer:
 
     def save(self, a: IDataset, b: IDataset):
         if len(a) != len(b):
-            print("Datasets have different lengths: %s vs %s" % \
-                (len(a), len(b)))
+            print("Datasets have different lengths: %s vs %s" % (len(a), len(b)))
 
         a_classes = a.categories().get(AnnotationType.label, LabelCategories())
         b_classes = b.categories().get(AnnotationType.label, LabelCategories())
-        class_mismatch = [(idx, a_cls, b_cls)
-            for idx, (a_cls, b_cls) in enumerate(
-                zip_longest(a_classes, b_classes))
-            if getattr(a_cls, 'name', None) != getattr(b_cls, 'name', None)
+        class_mismatch = [
+            (idx, a_cls, b_cls)
+            for idx, (a_cls, b_cls) in enumerate(zip_longest(a_classes, b_classes))
+            if getattr(a_cls, "name", None) != getattr(b_cls, "name", None)
         ]
         if class_mismatch:
             print("Datasets have mismatching labels:")
@@ -121,20 +124,23 @@ class DiffVisualizer:
             self.update_mask_confusion(mask_diff)
 
             self.save_item_label_diff(item_a, item_b, label_diff)
-            self.save_item_bbox_diff(item_a, item_b, bbox_diff)
+
+            if (
+                a.media_type()
+                and issubclass(a.media_type(), Image)
+                and b.media_type()
+                and issubclass(b.media_type(), Image)
+            ):
+                self.save_item_bbox_diff(item_a, item_b, bbox_diff)
 
         if len(self.label_confusion_matrix) != 0:
-            self.save_conf_matrix(self.label_confusion_matrix,
-                'label_confusion.png')
+            self.save_conf_matrix(self.label_confusion_matrix, "label_confusion.png")
         if len(self.bbox_confusion_matrix) != 0:
-            self.save_conf_matrix(self.bbox_confusion_matrix,
-                'bbox_confusion.png')
+            self.save_conf_matrix(self.bbox_confusion_matrix, "bbox_confusion.png")
         if len(self.polygon_confusion_matrix) != 0:
-            self.save_conf_matrix(self.polygon_confusion_matrix,
-                'polygon_confusion.png')
+            self.save_conf_matrix(self.polygon_confusion_matrix, "polygon_confusion.png")
         if len(self.mask_confusion_matrix) != 0:
-            self.save_conf_matrix(self.mask_confusion_matrix,
-                'mask_confusion.png')
+            self.save_conf_matrix(self.mask_confusion_matrix, "mask_confusion.png")
 
     def update_label_confusion(self, label_diff):
         matches, a_unmatched, b_unmatched = label_diff
@@ -167,33 +173,47 @@ class DiffVisualizer:
         self._update_segment_confusion(self.mask_confusion_matrix, diff)
 
     @classmethod
-    def draw_text_with_background(cls, frame, text, origin,
-            font=None, scale=1.0,
-            color=(0, 0, 0), thickness=1, bgcolor=(1, 1, 1)):
+    def draw_text_with_background(
+        cls,
+        frame,
+        text,
+        origin,
+        font=None,
+        scale=1.0,
+        color=(0, 0, 0),
+        thickness=1,
+        bgcolor=(1, 1, 1),
+    ):
         if not font:
             font = cv2.FONT_HERSHEY_SIMPLEX
 
         text_size, baseline = cv2.getTextSize(text, font, scale, thickness)
-        cv2.rectangle(frame,
+        cv2.rectangle(
+            frame,
             tuple((origin + (0, baseline)).astype(int)),
             tuple((origin + (text_size[0], -text_size[1])).astype(int)),
-            bgcolor, cv2.FILLED)
-        cv2.putText(frame, text,
-            tuple(origin.astype(int)),
-            font, scale, color, thickness)
+            bgcolor,
+            cv2.FILLED,
+        )
+        cv2.putText(frame, text, tuple(origin.astype(int)), font, scale, color, thickness)
         return text_size, baseline
 
     def draw_detection_roi(self, frame, x, y, w, h, label, conf, color):
         cv2.rectangle(frame, (x, y), (x + w, y + h), color, 2)
 
-        text = '%s %.2f%%' % (label, 100.0 * conf)
+        text = "%s %.2f%%" % (label, 100.0 * conf)
         text_scale = 0.5
         font = cv2.FONT_HERSHEY_SIMPLEX
         text_size = cv2.getTextSize(text, font, text_scale, 1)
         line_height = np.array([0, text_size[0][1]])
-        self.draw_text_with_background(frame, text,
+        self.draw_text_with_background(
+            frame,
+            text,
             np.array([x, y]) - line_height * 0.5,
-            font, scale=text_scale, color=[255 - c for c in color])
+            font,
+            scale=text_scale,
+            color=[255 - c for c in color],
+        )
 
     def get_a_label(self, label_id):
         return self._get_label(self._a_classes, label_id)
@@ -209,14 +229,15 @@ class DiffVisualizer:
 
     def draw_bbox(self, img, shape, label, color):
         x, y, w, h = shape.get_bbox()
-        self.draw_detection_roi(img, int(x), int(y), int(w), int(h),
-            label, shape.attributes.get('score', 1), color)
+        self.draw_detection_roi(
+            img, int(x), int(y), int(w), int(h), label, shape.attributes.get("score", 1), color
+        )
 
     def get_label_diff_file(self):
         if self._label_diff_writer is None:
             self._label_diff_writer = open(
-                osp.join(self._save_dir, 'label_diff.txt'),
-                'w', encoding='utf-8')
+                osp.join(self._save_dir, "label_diff.txt"), "w", encoding="utf-8"
+            )
         return self._label_diff_writer
 
     def save_item_label_diff(self, item_a, item_b, diff):
@@ -225,54 +246,47 @@ class DiffVisualizer:
         if 0 < len(a_unmatched) + len(b_unmatched):
             if self._output_format is self.OutputFormat.simple:
                 f = self.get_label_diff_file()
-                f.write(item_a.id + '\n')
+                f.write(item_a.id + "\n")
                 for a_label in a_unmatched:
-                    f.write('  >%s\n' % self.get_a_label(a_label))
+                    f.write("  >%s\n" % self.get_a_label(a_label))
                 for b_label in b_unmatched:
-                    f.write('  <%s\n' % self.get_b_label(b_label))
+                    f.write("  <%s\n" % self.get_b_label(b_label))
             elif self._output_format is self.OutputFormat.tensorboard:
                 tag = item_a.id
                 for a_label in a_unmatched:
-                    self._file_writer.add_text(tag,
-                        '>%s\n' % self.get_a_label(a_label))
+                    self._file_writer.add_text(tag, ">%s\n" % self.get_a_label(a_label))
                 for b_label in b_unmatched:
-                    self._file_writer.add_text(tag,
-                        '<%s\n' % self.get_b_label(b_label))
+                    self._file_writer.add_text(tag, "<%s\n" % self.get_b_label(b_label))
 
     def save_item_bbox_diff(self, item_a, item_b, diff):
         _, mispred, a_unmatched, b_unmatched = diff
 
         if 0 < len(a_unmatched) + len(b_unmatched) + len(mispred):
-            if not item_a.has_image or not item_a.image.has_data:
-                log.warning("Item %s: item has no image data, "
-                    "it will be skipped" % (item_a.id))
+            if not isinstance(item_a.media, Image) or not item_a.media.has_data:
+                log.warning("Item %s: item has no image data, " "it will be skipped" % (item_a.id))
                 return
-            img_a = item_a.image.data.copy()
+            img_a = item_a.media.data.copy()
             img_b = img_a.copy()
             for a_bbox, b_bbox in mispred:
-                self.draw_bbox(img_a, a_bbox, self.get_a_label(a_bbox.label),
-                    (0, 255, 0))
-                self.draw_bbox(img_b, b_bbox, self.get_b_label(b_bbox.label),
-                    (0, 0, 255))
+                self.draw_bbox(img_a, a_bbox, self.get_a_label(a_bbox.label), (0, 255, 0))
+                self.draw_bbox(img_b, b_bbox, self.get_b_label(b_bbox.label), (0, 0, 255))
             for a_bbox in a_unmatched:
-                self.draw_bbox(img_a, a_bbox, self.get_a_label(a_bbox.label),
-                    (255, 255, 0))
+                self.draw_bbox(img_a, a_bbox, self.get_a_label(a_bbox.label), (255, 255, 0))
             for b_bbox in b_unmatched:
-                self.draw_bbox(img_b, b_bbox, self.get_b_label(b_bbox.label),
-                    (255, 255, 0))
+                self.draw_bbox(img_b, b_bbox, self.get_b_label(b_bbox.label), (255, 255, 0))
 
             img = np.hstack([img_a, img_b])
 
             path = osp.join(self._save_dir, item_a.subset, item_a.id)
 
             if self._output_format is self.OutputFormat.simple:
-                save_image(path + '.png', img, create_dir=True)
+                save_image(path + ".png", img, create_dir=True)
             elif self._output_format is self.OutputFormat.tensorboard:
                 self.save_as_tensorboard(img, path)
 
     def save_as_tensorboard(self, img, name):
-        img = img[:, :, ::-1] # to RGB
-        img = np.transpose(img, (2, 0, 1)) # to (C, H, W)
+        img = img[:, :, ::-1]  # to RGB
+        img = np.transpose(img, (2, 0, 1))  # to (C, H, W)
         img = img.astype(dtype=np.uint8)
         self._file_writer.add_image(name, img)
 
@@ -282,18 +296,18 @@ class DiffVisualizer:
         def _get_class_map(label_categories):
             classes = None
             if label_categories is not None:
-                classes = { id: c.name
-                    for id, c in enumerate(label_categories.items) }
+                classes = {id: c.name for id, c in enumerate(label_categories.items)}
             if classes is None:
-                classes = { c: 'label_%s' % c for c, _ in conf_matrix }
-            classes[self._UNMATCHED_LABEL] = 'unmatched'
-            classes[None] = 'no_class'
+                classes = {c: "label_%s" % c for c, _ in conf_matrix}
+            classes[self._UNMATCHED_LABEL] = "unmatched"
+            classes[None] = "no_class"
             return classes
+
         a_classes = _get_class_map(self._a_classes)
         b_classes = _get_class_map(self._b_classes)
 
-        a_class_idx = { id: i for i, id in enumerate(a_classes) }
-        b_class_idx = { id: i for i, id in enumerate(b_classes) }
+        a_class_idx = {id: i for i, id in enumerate(a_classes)}
+        b_class_idx = {id: i for i, id in enumerate(b_classes)}
         matrix = np.zeros((len(a_classes), len(b_classes)), dtype=int)
         for idx_pair in conf_matrix:
             index = (a_class_idx[idx_pair[0]], b_class_idx[idx_pair[1]])
@@ -304,20 +318,14 @@ class DiffVisualizer:
 
         fig = plt.figure()
         fig.add_subplot(111)
-        table = plt.table(
-            cellText=matrix,
-            rowLabels=a_labels,
-            colLabels=b_labels,
-            loc='center')
+        table = plt.table(cellText=matrix, rowLabels=a_labels, colLabels=b_labels, loc="center")
         table.auto_set_font_size(False)
         table.set_fontsize(8)
         table.scale(3, 3)
         # Removing ticks and spines enables you to get the figure only with table
-        plt.tick_params(axis='x', which='both',
-            bottom=False, top=False, labelbottom=False)
-        plt.tick_params(axis='y', which='both',
-            right=False, left=False, labelleft=False)
-        for pos in ['right','top','bottom','left']:
+        plt.tick_params(axis="x", which="both", bottom=False, top=False, labelbottom=False)
+        plt.tick_params(axis="y", which="both", right=False, left=False, labelleft=False)
+        for pos in ["right", "top", "bottom", "left"]:
             plt.gca().spines[pos].set_visible(False)
 
         for idx_pair in conf_matrix:
@@ -325,9 +333,8 @@ class DiffVisualizer:
             j = b_class_idx[idx_pair[1]]
             if conf_matrix[idx_pair] != 0:
                 if a_classes[idx_pair[0]] == b_classes[idx_pair[1]]:
-                    table._cells[(i + 1, j)].set_facecolor('#00FF00')
+                    table._cells[(i + 1, j)].set_facecolor("#00FF00")
                 else:
-                    table._cells[(i + 1, j)].set_facecolor('#FF0000')
+                    table._cells[(i + 1, j)].set_facecolor("#FF0000")
 
-        plt.savefig(osp.join(self._save_dir, filename),
-            bbox_inches='tight', pad_inches=0.05)
+        plt.savefig(osp.join(self._save_dir, filename), bbox_inches="tight", pad_inches=0.05)
