@@ -9,14 +9,15 @@ class CocoTask(Enum):
     instances = auto()
     person_keypoints = auto()
     captions = auto()
-    labels = auto() # extension, does not exist in the original COCO format
+    labels = auto()  # extension, does not exist in the original COCO format
     image_info = auto()
     panoptic = auto()
     stuff = auto()
 
-class CocoPath:
-    IMAGES_DIR = 'images'
-    ANNOTATIONS_DIR = 'annotations'
 
-    IMAGE_EXT = '.jpg'
-    PANOPTIC_EXT = '.png'
+class CocoPath:
+    IMAGES_DIR = "images"
+    ANNOTATIONS_DIR = "annotations"
+
+    IMAGE_EXT = ".jpg"
+    PANOPTIC_EXT = ".png"
