@@ -3,12 +3,12 @@
 # SPDX-License-Identifier: MIT
 
 from itertools import groupby
+from typing import Callable, Dict, Iterable, NewType, Optional, Sequence, Tuple, Union
 
 import numpy as np
+from typing_extensions import Literal
 
-from datumaro.components.extractor import (
-    AnnotationType, LabelCategories, Mask, RleMask, _Shape,
-)
+from datumaro.components.annotation import AnnotationType, LabelCategories, Mask, RleMask, _Shape
 from datumaro.util.mask_tools import mask_to_rle
 
 
@@ -23,16 +23,33 @@ def find_instances(instance_anns):
 
     return ann_groups
 
+
 def find_group_leader(group):
     return max(group, key=lambda x: x.get_area())
 
-def _get_bbox(ann):
+
+BboxCoords = Tuple[float, float, float, float]
+Shape = NewType("Shape", _Shape)
+SpatialAnnotation = Union[Shape, Mask]
+
+
+def _get_bbox(ann: Union[Sequence, SpatialAnnotation]) -> BboxCoords:
     if isinstance(ann, (_Shape, Mask)):
         return ann.get_bbox()
-    else:
+    elif hasattr(ann, "__len__") and len(ann) == 4:
         return ann
+    else:
+        raise ValueError("The value of type '%s' can't be treated as a " "bounding box" % type(ann))
+
+
+def max_bbox(annotations: Iterable[Union[BboxCoords, SpatialAnnotation]]) -> BboxCoords:
+    """
+    Computes the maximum bbox for the set of spatial annotations and boxes.
+
+    Returns:
+      bbox (tuple): (x, y, w, h)
+    """
 
-def max_bbox(annotations):
     boxes = [_get_bbox(ann) for ann in annotations]
     x0 = min((b[0] for b in boxes), default=0)
     y0 = min((b[1] for b in boxes), default=0)
@@ -40,7 +57,15 @@ def max_bbox(annotations):
     y1 = max((b[1] + b[3] for b in boxes), default=0)
     return [x0, y0, x1 - x0, y1 - y0]
 
-def mean_bbox(annotations):
+
+def mean_bbox(annotations: Iterable[Union[BboxCoords, SpatialAnnotation]]) -> BboxCoords:
+    """
+    Computes the mean bbox for the set of spatial annotations and boxes.
+
+    Returns:
+      bbox (tuple): (x, y, w, h)
+    """
+
     le = len(annotations)
     boxes = [_get_bbox(ann) for ann in annotations]
     mlb = sum(b[0] for b in boxes) / le
@@ -49,15 +74,17 @@ def mean_bbox(annotations):
     mbb = sum(b[1] + b[3] for b in boxes) / le
     return [mlb, mtb, mrb - mlb, mbb - mtb]
 
+
 def softmax(x):
     return np.exp(x) / sum(np.exp(x))
 
+
 def nms(segments, iou_thresh=0.5):
     """
     Non-maxima suppression algorithm.
     """
 
-    indices = np.argsort([b.attributes['score'] for b in segments])
+    indices = np.argsort([b.attributes["score"] for b in segments])
     ious = np.array([[segment_iou(a, b) for b in segments] for a in segments])
 
     predictions = []
@@ -73,7 +100,8 @@ def nms(segments, iou_thresh=0.5):
 
     return predictions
 
-def bbox_iou(a, b):
+
+def bbox_iou(a, b) -> Union[Literal[-1], float]:
     """
     IoU computations for simple cases with bounding boxes
     """
@@ -98,6 +126,7 @@ def bbox_iou(a, b):
     union = a_area + b_area - intersection
     return intersection / union
 
+
 def segment_iou(a, b):
     """
     Generic IoU computation with masks, polygons, and boxes.
@@ -105,8 +134,8 @@ def segment_iou(a, b):
     """
     from pycocotools import mask as mask_utils
 
-    a_bbox = a.get_bbox()
-    b_bbox = b.get_bbox()
+    a_bbox = list(a.get_bbox())
+    b_bbox = list(b.get_bbox())
 
     is_bbox = AnnotationType.bbox in [a.type, b.type]
     if is_bbox:
@@ -125,10 +154,12 @@ def segment_iou(a, b):
                 return mask_utils.frPyObjects([mask_to_rle(ann.image)], h, w)
             else:
                 raise TypeError("Unexpected arguments: %s, %s" % (a, b))
+
         a = _to_rle(a)
         b = _to_rle(b)
     return float(mask_utils.iou(a, b, [not is_bbox]))
 
+
 def PDJ(a, b, eps=None, ratio=0.05, bbox=None):
     """
     Percentage of Detected Joints metric.
@@ -152,6 +183,7 @@ def PDJ(a, b, eps=None, ratio=0.05, bbox=None):
     dists = np.linalg.norm(p1 - p2, axis=1)
     return np.sum(dists < eps) / len(p1)
 
+
 def OKS(a, b, sigma=None, bbox=None, scale=None):
     """
     Object Keypoint Similarity metric.
@@ -174,13 +206,27 @@ def OKS(a, b, sigma=None, bbox=None, scale=None):
         scale = bbox[2] * bbox[3]
 
     dists = np.linalg.norm(p1 - p2, axis=1)
-    return np.sum(np.exp(-(dists ** 2) / (2 * scale * (2 * sigma) ** 2)))
+    return np.sum(np.exp(-(dists**2) / (2 * scale * (2 * sigma) ** 2)))
+
+
+def approximate_line(points: Sequence[float], segments: int) -> np.ndarray:
+    """
+    Approximates a 2d line to the required number of segments. The new points
+    are distributed uniformly across the input line.
+
+    Args:
+      points (Sequence): an array of line point coordinates.
+        The size is [points * 2], the layout is [x0, y0, x1, y1, ...].
+      segments (int): the required numebr of segments in the resulting line.
+
+    Returns:
+        new_points (numpy.ndarray): an array of new line point coordinates.
+    Returns:
+        The size is [(segments + 1) * 2], the layout is [x0, y0, x1, y1, ...].
+    """
 
-def smooth_line(points, segments):
     assert 2 <= len(points) // 2 and len(points) % 2 == 0
-
-    if len(points) // 2 == segments:
-        return points
+    assert 0 < segments
 
     points = list(points)
     if len(points) == 2:
@@ -211,20 +257,32 @@ def smooth_line(points, segments):
 
         new_points[new_segment] = prev_p * (1 - r) + next_p * r
 
-    return new_points, step
+    return np.reshape(new_points, (-1,))
+
 
 def make_label_id_mapping(
-        src_labels: LabelCategories, dst_labels: LabelCategories, fallback=0):
-    source_labels = { id: label.name
-        for id, label in enumerate(src_labels or LabelCategories().items)
-    }
-    target_labels = { label.name: id
-        for id, label in enumerate(dst_labels or LabelCategories().items)
-    }
-    id_mapping = { src_id: target_labels.get(src_label, fallback)
+    src_labels: LabelCategories, dst_labels: LabelCategories, fallback: int = 0
+) -> Tuple[Callable[[int], Optional[int]], Dict[int, int], Dict[int, str], Dict[int, str]]:
+    """
+    Maps label ids from source to destination. Fallback id is used for missing
+    labels.
+
+    Returns:
+
+    |   map_id (callable): src id -> dst id
+    |   id_mapping (dict): src id -> dst i
+    |   src_labels (dict): src id -> src label
+    |   dst_labels (dict): dst id -> dst label
+    """
+
+    source_labels = {id: label.name for id, label in enumerate(src_labels or ())}
+    target_labels = {label.name: id for id, label in enumerate(dst_labels or ())}
+    id_mapping = {
+        src_id: target_labels.get(src_label, fallback)
         for src_id, src_label in source_labels.items()
     }
 
     def map_id(src_id):
         return id_mapping.get(src_id, fallback)
+
     return map_id, id_mapping, source_labels, target_labels
