@@ -2,17 +2,23 @@
 #
 # SPDX-License-Identifier: MIT
 
+from functools import partial
 from itertools import chain
+from typing import Tuple
 
 import numpy as np
 
 from datumaro.util.image import lazy_image, load_image
 
 
-def generate_colormap(length=256):
+def generate_colormap(length=256, *, include_background=True):
     """
     Generates colors using PASCAL VOC algorithm.
 
+    If include_background is True, the result will include the item
+    "0: (0, 0, 0)", which is typically used as a background color.
+    Otherwise, indices will start from 0, but (0, 0, 0) is not included.
+
     Returns index -> (R, G, B) mapping.
     """
 
@@ -20,64 +26,72 @@ def generate_colormap(length=256):
         return (number >> index) & 1
 
     colormap = np.zeros((length, 3), dtype=int)
-    indices = np.arange(length, dtype=int)
+
+    offset = int(not include_background)
+    indices = np.arange(offset, length + offset, dtype=int)
 
     for j in range(7, -1, -1):
         for c in range(3):
             colormap[:, c] |= get_bit(indices, c) << j
         indices >>= 3
 
-    return {
-        id: tuple(color) for id, color in enumerate(colormap)
-    }
+    return {id: tuple(color) for id, color in enumerate(colormap)}
+
 
 def invert_colormap(colormap):
-    return {
-        tuple(a): index for index, a in colormap.items()
-    }
+    return {tuple(a): index for index, a in colormap.items()}
+
 
 def check_is_mask(mask):
     assert len(mask.shape) in {2, 3}
     if len(mask.shape) == 3:
         assert mask.shape[2] == 1
 
+
 _default_colormap = generate_colormap()
 _default_unpaint_colormap = invert_colormap(_default_colormap)
 
-def unpaint_mask(painted_mask, inverse_colormap=None):
-    # Convert color mask to index mask
 
-    # mask: HWC BGR [0; 255]
-    # colormap: (R, G, B) -> index
+def unpaint_mask(painted_mask, inverse_colormap=None):
+    """
+    Convert color mask to index mask
+
+    mask: HWC BGR [0; 255]
+
+    colormap: (R, G, B) -> index
+    """
     assert len(painted_mask.shape) == 3
     if inverse_colormap is None:
         inverse_colormap = _default_unpaint_colormap
 
     if callable(inverse_colormap):
-        map_fn = lambda a: inverse_colormap(
-                (a >> 16) & 255, (a >> 8) & 255, a & 255
-            )
+        map_fn = lambda a: inverse_colormap((a >> 16) & 255, (a >> 8) & 255, a & 255)
     else:
-        map_fn = lambda a: inverse_colormap[(
-                (a >> 16) & 255, (a >> 8) & 255, a & 255
-            )]
+        map_fn = lambda a: inverse_colormap.get(((a >> 16) & 255, (a >> 8) & 255, a & 255), None)
 
     painted_mask = painted_mask.astype(int)
-    painted_mask = painted_mask[:, :, 0] + \
-                   (painted_mask[:, :, 1] << 8) + \
-                   (painted_mask[:, :, 2] << 16)
+    painted_mask = (
+        painted_mask[:, :, 0] + (painted_mask[:, :, 1] << 8) + (painted_mask[:, :, 2] << 16)
+    )
     uvals, unpainted_mask = np.unique(painted_mask, return_inverse=True)
-    palette = np.array([map_fn(v) for v in uvals],
-        dtype=np.min_scalar_type(len(uvals)))
+    palette = []
+    for v in uvals:
+        class_id = map_fn(v)
+        if class_id is None:
+            raise KeyError(f"Undeclared color {((v >> 16) & 255, (v >> 8) & 255, v & 255)}")
+        palette.append(class_id)
+    palette = np.array(palette, dtype=np.min_scalar_type(len(uvals)))
     unpainted_mask = palette[unpainted_mask].reshape(painted_mask.shape[:2])
 
     return unpainted_mask
 
+
 def paint_mask(mask, colormap=None):
     """
     Applies colormap to index mask
 
     mask: HW(C) [0; max_index] mask
+
     colormap: index -> (R, G, B)
     """
     check_is_mask(mask)
@@ -94,6 +108,7 @@ def paint_mask(mask, colormap=None):
     painted_mask = palette[mask].reshape((*mask.shape[:2], 3))
     return painted_mask
 
+
 def remap_mask(mask, map_fn):
     """
     Changes mask elements from one colormap to another
@@ -104,23 +119,27 @@ def remap_mask(mask, map_fn):
 
     return np.array([map_fn(c) for c in range(256)], dtype=np.uint8)[mask]
 
+
 def make_index_mask(binary_mask, index, dtype=None):
-    return binary_mask * np.array([index],
-        dtype=dtype or np.min_scalar_type(index))
+    return binary_mask * np.array([index], dtype=dtype or np.min_scalar_type(index))
+
 
 def make_binary_mask(mask):
-    if mask.dtype.kind == 'b':
+    if mask.dtype.kind == "b":
         return mask
     return mask.astype(bool)
 
+
 def bgr2index(img):
-    if img.dtype.kind not in {'b', 'i', 'u'} or img.dtype.itemsize < 4:
+    if img.dtype.kind not in {"b", "i", "u"} or img.dtype.itemsize < 4:
         img = img.astype(np.uint32)
     return (img[..., 0] << 16) + (img[..., 1] << 8) + img[..., 2]
 
+
 def index2bgr(id_map):
     return np.dstack((id_map >> 16, id_map >> 8, id_map)).astype(np.uint8)
 
+
 def load_mask(path, inverse_colormap=None):
     mask = load_image(path, dtype=np.uint8)
     if inverse_colormap is not None:
@@ -128,17 +147,19 @@ def load_mask(path, inverse_colormap=None):
             mask = unpaint_mask(mask, inverse_colormap)
     return mask
 
+
 def lazy_mask(path, inverse_colormap=None):
-    return lazy_image(path, lambda path: load_mask(path, inverse_colormap))
+    return lazy_image(path, partial(load_mask, inverse_colormap=inverse_colormap))
+
 
 def mask_to_rle(binary_mask):
     # walk in row-major order as COCO format specifies
-    bounded = binary_mask.ravel(order='F')
+    bounded = binary_mask.ravel(order="F")
 
     # add borders to sequence
     # find boundary positions for sequences and compute their lengths
     difs = np.diff(bounded, prepend=[1 - bounded[0]], append=[1 - bounded[-1]])
-    counts, = np.where(difs != 0)
+    (counts,) = np.where(difs != 0)
 
     # start RLE encoding from 0 as COCO format specifies
     if bounded[0] != 0:
@@ -146,10 +167,8 @@ def mask_to_rle(binary_mask):
     else:
         counts = np.diff(counts)
 
-    return {
-        'counts': counts,
-        'size': list(binary_mask.shape)
-    }
+    return {"counts": counts, "size": list(binary_mask.shape)}
+
 
 def mask_to_polygons(mask, area_threshold=1):
     """
@@ -164,13 +183,14 @@ def mask_to_polygons(mask, area_threshold=1):
     Returns:
         A list of polygons like [[x1,y1, x2,y2 ...], [...]]
     """
-    from pycocotools import mask as mask_utils
     import cv2
+    from pycocotools import mask as mask_utils
 
     polygons = []
 
-    contours, _ = cv2.findContours(mask.astype(np.uint8),
-        mode=cv2.RETR_TREE, method=cv2.CHAIN_APPROX_TC89_KCOS)
+    contours, _ = cv2.findContours(
+        mask.astype(np.uint8), mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_TC89_KCOS
+    )
 
     for contour in contours:
         if len(contour) <= 2:
@@ -179,8 +199,8 @@ def mask_to_polygons(mask, area_threshold=1):
         contour = contour.reshape((-1, 2))
 
         if not np.array_equal(contour[0], contour[-1]):
-            contour = np.vstack((contour, contour[0])) # make polygon closed
-        contour = contour.flatten().clip(0) # [x0, y0, ...]
+            contour = np.vstack((contour, contour[0]))  # make polygon closed
+        contour = contour.flatten().clip(0)  # [x0, y0, ...]
 
         # Check if the polygon is big enough
         rle = mask_utils.frPyObjects([contour], mask.shape[0], mask.shape[1])
@@ -189,9 +209,16 @@ def mask_to_polygons(mask, area_threshold=1):
             polygons.append(contour)
     return polygons
 
-def crop_covered_segments(segments, width, height,
-        iou_threshold=0.0, ratio_tolerance=0.001, area_threshold=1,
-        return_masks=False):
+
+def crop_covered_segments(
+    segments,
+    width,
+    height,
+    iou_threshold=0.0,
+    ratio_tolerance=0.001,
+    area_threshold=1,
+    return_masks=False,
+):
     """
     Find all segments occluded by others and crop them to the visible part only.
     Input segments are expected to be sorted from background to foreground.
@@ -209,6 +236,9 @@ def crop_covered_segments(segments, width, height,
 
     Returns:
         A list of input segments' parts (in the same order as input):
+
+        .. code-block::
+
             [
                 [[x1,y1, x2,y2 ...], ...], # input segment #0 parts
                 mask1, # input segment #1 mask (if source segment is mask)
@@ -230,7 +260,7 @@ def crop_covered_segments(segments, width, height,
         rles_top = []
         for j in range(i + 1, len(input_rles)):
             rle_top = input_rles[j]
-            iou = sum(mask_utils.iou(rle_bottom, rle_top, [0, 0]))[0]
+            iou = sum(mask_utils.iou(rle_bottom, rle_top, [0]))[0]
 
             if iou <= iou_threshold:
                 continue
@@ -250,8 +280,7 @@ def crop_covered_segments(segments, width, height,
 
             rles_top += rle_top
 
-        if not rles_top and not isinstance(segments[i][0], dict) \
-                and not return_masks:
+        if not rles_top and not isinstance(segments[i][0], dict) and not return_masks:
             continue
 
         rle_bottom = rle_bottom[0]
@@ -265,13 +294,13 @@ def crop_covered_segments(segments, width, height,
             bottom_mask[bottom_mask != 1] = 0
 
         if not return_masks and not isinstance(segments[i][0], dict):
-            segments[i] = mask_to_polygons(bottom_mask,
-                area_threshold=area_threshold)
+            segments[i] = mask_to_polygons(bottom_mask, area_threshold=area_threshold)
         else:
             segments[i] = bottom_mask
 
     return segments
 
+
 def rles_to_mask(rles, width, height):
     from pycocotools import mask as mask_utils
 
@@ -280,21 +309,24 @@ def rles_to_mask(rles, width, height):
     mask = mask_utils.decode(rles)
     return mask
 
-def find_mask_bbox(mask):
+
+def find_mask_bbox(mask) -> Tuple[int, int, int, int]:
     cols = np.any(mask, axis=0)
     rows = np.any(mask, axis=1)
     x0, x1 = np.where(cols)[0][[0, -1]]
     y0, y1 = np.where(rows)[0][[0, -1]]
-    return [x0, y0, x1 - x0, y1 - y0]
+    return (x0, y0, x1 - x0, y1 - y0)
+
 
 def merge_masks(masks, start=None):
     """
-        Merges masks into one, mask order is responsible for z order.
-        To avoid memory explosion on mask materialization, consider passing
-        a generator.
+    Merges masks into one, mask order is responsible for z order.
+    To avoid memory explosion on mask materialization, consider passing
+    a generator.
 
-        Inputs: a sequence of index masks or (binary mask, index) pairs
-        Outputs: an index mask
+    Inputs: a sequence of index masks or (binary mask, index) pairs
+
+    Outputs: an index mask
     """
     if start is not None:
         masks = chain([start], masks)
